#######################
## ABI functions...
#######################
require(maDB)

getAssayNames <- function(file,check.names=FALSE){
	head <- read.table(file,sep="\t",as.is=TRUE,header=TRUE,nrows=2,check.names=check.names)
	columns <- colnames(head)
	assays <- columns[grep(columns,pattern="_ASSAY_NAME")]
#	ret <- replaceAll(assays,old="_ASSAY_NAME",new="")
	ret <- gsub("_ASSAY_NAME","",assays)
	ret
}


readABIData <- function(files,assays=NULL,signal="SIGNAL",v=TRUE,flagabove=100,check.names=FALSE){
	if(length(files)==1){
		if(!is.null(assays)){
			if(!class(assays)=="list"){
				assays <- list(assays)
			}
		}
		else{
			assays <- list(assays)
		}
	}

    ## need these data:
    E <- NULL       ## the big expression data...
    W <- NULL       ## weights...
    G <- NULL       ## genes...

	for(i in 1:length(files)){
		if(i==1){
			eset <- .readABI2(files[i],assays=assays[[i]],signal=signal,v=v,flagabove=flagabove,check.names=check.names)
            E <- exprs(eset)
            W <- getWeights(eset)
            G <- eset@genes
            CN <- colnames(E)
		}
		else{
			dummy <- .readABI2(files[i],assays=assays[[i]],signal=signal,v=v,flagabove=flagabove,check.names=check.names)
            ## check if the microarrays are the same...
            if(nrow(E)!=nrow(exprs(dummy))){
                stop("ABI files contain different number of features! Cannot process data from different ABI microarrays!\n")
            }
            ## check for the correct ordering!
            if(sum(G[,"GENE_ID"]==dummy@genes[,"GENE_ID"])!=nrow(G)){
                stop("The ordering of the data in the ABI files is not the same! Cannot process data from different ABI microarrays or from differently ordered data!\n")
            }
            if(sum(colnames(G)=="PROBE_ID")==1 & sum(colnames(dummy@genes)=="PROBE_ID")==1){
                if(sum(G[,"PROBE_ID"]==dummy@genes[,"PROBE_ID"])!=nrow(G)){
                    stop("The ordering of the data in the ABI files is not the same! Cannot process data from different ABI microarrays or from differently ordered data!\n")
                }
            }
            ## check if the assay names are the same...
            if(sum( CN %in% sampleNames(dummy) ) > 0){
                    stop("Some assay names are identical in the input ABI files! Please do not submit the same ABI file twice!\n")
            }

            ## passed all checks... collating data...
            CN <- c(CN,sampleNames(dummy))
            W <- cbind(W,getWeights(dummy))
            colnames(W) <- CN
            E <- cbind(E,exprs(dummy))
            colnames(E) <- CN
		}
	}
    ## generating a new MadbSet...
    ## generating phenoData of the two data sets (ExpressionSet classes).
    dummy3 <- data.frame(sample=as.character(1:length(CN)))
    rownames(dummy3) <- CN
    otherdummy <- data.frame(labelDescription=c("Arbitrary numbering"))
    rownames(otherdummy) <- c("sample")
    eset <- new("MadbSet",exprs=E,weights=W,genes=G,phenoData=new("AnnotatedDataFrame",data=dummy3,varMetadata=otherdummy))
    return(eset)
}

flag <- function(x,level){
	if(x>=level){
		ret <- 0
	}
	else{
		ret <- 1
	}
	ret
}

.readABI <- function(file,assays=NULL,signal="SIGNAL",v=TRUE,flagabove=100,check.names=FALSE){
	dummy <- read.table(file,sep="\t",as.is=TRUE,header=TRUE,check.names=check.names)
	if(is.null(assays)){
		if(v){
			cat("Reading all assays from the file ",file,"\n")
		}
		assays <- getAssayNames(file,check.names=check.names)
	}
	eset <- new("EexprSet")
#	cat(assays,"\n")
#	cat(paste(assays,"_",signal,sep=""))
	dummy2 <- dummy[,paste(assays,"_",signal,sep="")]
	dummy2 <- apply(dummy2,c(1,2),gsub,pattern=",",replacement="")
	dummy2 <- apply(dummy2,c(1,2),as.numeric)
	eset@exprs <- as.matrix(dummy2)
	rm(dummy2)
	colnames(eset@exprs) <- assays
	rownames(eset@exprs) <- dummy[,"GENE_ID"]
	c.names <- "GENE_ID"
	if(sum(colnames(dummy)=="PROBE_ID")>0){
		c.names <- c(c.names,"PROBE_ID")
		#cat(length(dummy[,"PROBE_ID"]),length(unique(dummy[,"PROBE_ID"])),"\n")
	}
	eset@genes <- as.matrix(dummy[,c.names])
	colnames(eset@genes) <- c.names
	
	##
	# flagging
	if(!is.null(flagabove)){
		weights <- dummy[,paste(assays,"_FLAGS",sep="")]
		weights <- apply(weights,c(1,2),gsub,pattern=",",replacement="")
		weights <- apply(weights,c(1,2),as.numeric)
		rownames(weights) <- dummy[,"GENE_ID"]
		colnames(weights) <- assays
		weights <- apply(weights,c(1,2),flag,level=flagabove)
		eset@weights <- as.matrix(weights)
	}
	eset
}


###################
## same as .readABI, but returns a MadbSet instead of an EexprSet
.readABI2 <- function(file,assays=NULL,signal="SIGNAL",v=TRUE,flagabove=100,check.names=FALSE){
    dummy <- read.table(file,sep="\t",as.is=TRUE,header=TRUE,check.names=check.names)
    ng <- is.na(dummy[,"GENE_ID"])
    dummy <- dummy[!ng,]
    ng <- dummy[,"GENE_ID"]==""
    dummy <- dummy[!ng,]
    if(is.null(assays)){
        if(v){
            cat("Reading all assays from the file ",file,"\n")
        }
        assays <- getAssayNames(file,check.names=check.names)
    }
    eset <- new("MadbSet")

    ## generating phenoData of the two data sets (ExpressionSet classes).
    dummy3 <- data.frame(sample=as.character(1:length(assays)))
    rownames(dummy3) <- assays
    otherdummy <- data.frame(labelDescription=c("Arbitrary numbering"))
    rownames(otherdummy) <- c("sample")

#   cat(assays,"\n")
#   cat(paste(assays,"_",signal,sep=""))
    dummy2 <- dummy[,paste(assays,"_",signal,sep="")]
    dummy2 <- apply(dummy2,c(1,2),gsub,pattern=",",replacement="")
    dummy2 <- apply(dummy2,c(1,2),as.numeric)
    dummy2 <- as.matrix(dummy2)
    colnames(dummy2) <- assays
    if(length(unique(dummy[,"GENE_ID"]))==nrow(dummy2)){
        rownames(dummy2) <- dummy[,"GENE_ID"]
    }
    else{
        rownames(dummy2) <- as.character(1:nrow(dummy2))
    }

    c.names <- "GENE_ID"
    if(sum(colnames(dummy)=="PROBE_ID")>0){
        c.names <- c(c.names,"PROBE_ID")
        #cat(length(dummy[,"PROBE_ID"]),length(unique(dummy[,"PROBE_ID"])),"\n")
    }

    Genes <- as.matrix(dummy[,c.names])
    colnames(Genes) <- c.names
    if(length(unique(Genes[,"GENE_ID"]))==nrow(Genes)){
        rownames(Genes) <- Genes[,"GENE_ID"]
    }
    else{
        rownames(Genes) <- as.character(1:nrow(Genes))
        Genes <- cbind(idx=as.character(1:nrow(Genes)),Genes)
    }

    eset <- new("MadbSet",exprs=dummy2, phenoData=new("AnnotatedDataFrame",data=dummy3,varMetadata=otherdummy), genes=Genes)

    rm(Genes)
    rm(dummy2)

    ##
    # flagging
    if(!is.null(flagabove)){
        weights <- dummy[,paste(assays,"_FLAGS",sep="")]
        weights <- apply(weights,c(1,2),gsub,pattern=",",replacement="")
        weights <- apply(weights,c(1,2),as.numeric)
        rownames(weights) <- featureNames(eset)
        colnames(weights) <- assays
        weights <- apply(weights,c(1,2),flag,level=flagabove)
        eset@weights <- as.matrix(weights)
    }
    rm(dummy)

    eset
}





